from pathlib import Path
import os
import flask

from .model import Photo

app: flask.Flask = flask.current_app
blueprint = flask.Blueprint('photo', __name__, url_prefix='/photos')


@blueprint.route('<path:path>.tiff')
@blueprint.route('<path:path>.tif')
def image_tiff(path: str):
    """
    /photos/<session>/<set>/<device id>/<camera id>/<timestamp>_<photo id>.tiff
    """
    photo = Photo(path + ".np")

    # TIFF image data
    return flask.send_file(photo.to_tiff(), mimetype='image/tiff', as_attachment=False)


@blueprint.route('<path:path>.png')
def image_png(path: str):
    """
    /photos/<session>/<set>/<device id>/<camera id>/<timestamp>_<photo id>.png
    """
    photo = Photo(path + ".np")

    # Return the image as a PNG file
    return flask.send_file(photo.to_png(), mimetype='image/png', as_attachment=False)


@blueprint.route('/dimension', methods=['GET'])
def image_dimension():  # to get the width and height of the image for initialisation of imageSize in the Image.jsx
    photo_path = flask.request.args['path']
    photo = Photo(photo_path)
    return flask.jsonify(photo.dimension)


@blueprint.route('<path:path>.jpeg')
@blueprint.route('<path:path>.jpg')
def image_jpeg(path: str):
    photo = Photo(path + ".np")

    # Return the image as a PNG file
    return flask.send_file(photo.to_jpeg(), mimetype='image/jpeg', as_attachment=False)


@blueprint.route('/<path:path>.json')
def detail(path: str):
    """
    Show the photo metadata

    Usage:
    /photos/1970-01-01/set_A/device_1234/camera_1/20200101_094359.123456_000002.json

    :param path: The path of the data file (but with a JSON file extension)
    :returns: The metadata for that photo as a JSON object.
    """
    photo = Photo(path + '.np')
    return flask.jsonify(photo.metadata)


@blueprint.route('/next')
def next_():
    """
    Get the path of the next photo in the collection directory.


    Usage:
    /photos/next/?path=1970-01-01/set_A/device_1234/camera_1/20200101_094359.123456_000002.np&skip=-10
    """
    root_directory = Photo.root_directory()
    path = flask.request.args['path']
    skip = int(flask.request.args.get('skip', 1))
    photo = Photo(path)
    next_photo_path = photo.next(skip=skip)
    filename = next_photo_path.relative_to(root_directory)
    return flask.jsonify(str(filename))



import collections.abc
import dataclasses
from pathlib import Path
from typing import Generator, Tuple, TypeVar, Union

import flask

from btviewer.mixins import StructureMixin

app = flask.current_app

#Declare a generic type variable
Session = TypeVar('Session')


class Session(StructureMixin):
    """
    A photo data gathering session.
    """

    def __init__(self, session_id: Union[str, Path]):
        if session_id is None:
            raise ValueError(session_id)
        if isinstance(session_id, Path):
            self.path_to_session_id(session_id)
        self.id = str(session_id)

    @classmethod
    def path_to_session_id(cls, path: Path) -> str:
        """
        If a path is passed in, then use the directory name
        """
        if not path.exists():
            raise FileNotFoundError(path)
        if not path.is_dir():
            raise NotADirectoryError(path)
        return path.name

    @classmethod
    def root_directory(cls) -> Path:
        return Path(app.config['ROOT_DIRECTORY']).absolute()

    @property
    def path(self) -> Path:
        """
        The path of this session's directory.
        e.g. ~/my_data/my_session_1
        """
        return self.root_directory().joinpath(self.id)

    def iter_set_paths(self) -> Generator[Path, None, None]:
        """
        Get all the photo sets in this session by iterating over
        all the files in this directory.
        """
        for path in self.path.iterdir():
            if path.is_dir():
                yield path

    @classmethod
    def iter_session_paths(cls) -> Generator[Path, None, None]:
        """
        Iterate over all the paths of the available session directories.

        Usage:
        for path in Session.iter_session_paths():
            print(path)
        """
        for path in cls.root_directory().iterdir():
            if path.is_dir():
                yield path

    @classmethod
    def iter_sessions(cls) -> Generator[Session, None, None]:
        """
        Iterate over all the available sessions

        Usage:
        for session in Session.iter_sessions():
            print(session)
        """
        for path in cls.iter_session_paths():
            yield cls(path.name)



from http import HTTPStatus

import pytest

# Test data URLs
URLS = {
    '/sessions/list/',
    '/sessions/list/?path=2020-01-01/',
    '/sessions/list/?path=2020-01-01/set_A',
    '/sessions/list/?path=2020-01-01/set_A/device_1',
    '/sessions/list/?path=2020-01-01/set_A/device_1/camera_1',
    '/sessions/list/?path=2020-01-01/set_A/device_1/camera_1/20200101_094359.123456_000001.np',
}


@pytest.mark.parametrize('url', URLS)
def test_session_list(client, url):
    # Run the HTTP request
    response = client.get(url)
    assert response.status_code == HTTPStatus.OK

    # Check response contents
    # It should be a list of strings e.g.
    # ["session_1", "session_2"]
    sessions: list[str] = response.json
    assert isinstance(sessions, list)
    for session_id in sessions:
        assert isinstance(session_id, str)



from pathlib import Path
from collections.abc import Mapping

import json


class Label:
    """
    A class to represent a label
    """

    def __init__(self):
        raise NotImplementedError

    @classmethod
    def list(cls, directory_path: Path):
        """
        List all the labels in a directory
        """
        raise NotImplementedError

    @property
    def path(self) -> Path:
        """
        The file path of the label file.
        :return:
        """
        raise NotImplementedError

    def save(self):
        """
        Store the label to disk
        """
        raise NotImplementedError

    def to_dict(self) -> Mapping:
        """
        Convert the label to a dictionary
        """
        raise NotImplementedError

    def to_json(self) -> str:
        return json.dumps(self.to_dict())



from pathlib import Path

import flask

from .models import Session

app: flask.Flask = flask.current_app

blueprint = flask.Blueprint('session', __name__, url_prefix='/sessions')


@blueprint.route("/list/")
def list_():
    """
    List directory contents of _any_ data subdirectory

    <root_directory>/<session>/<set>/<device id>/<camera id>/<timestamp>_<photo id>.np

    This endpoint returns a list of strings, each is the path of that directory or photo file.

    Example: GET /sessions/list/?path=2020-01-01/set_A/device_1
    [
      "2020-01-01/set_A/device_1/camera_1",
      "2020-01-01/set_A/device_1/camera_2"
    ]
    """
    # Get the specified directory
    relative_path = Path(flask.request.args.get('path', ''))
    root_directory = Session.root_directory()
    path: Path = root_directory.joinpath(relative_path)

    # Are we in a camera directory that contains photos?
    # (i.e. Are we at least four subdirectories deep in the file structure?)
    if len(relative_path.parents) >= 4:
        # Get photo files
        paths = path.glob("*.np")
    else:
        # Get subdirectory names
        paths = (_path for _path in path.iterdir() if _path.is_dir())

    # Get the full path relative to the root directory
    full_paths = tuple(str(path.relative_to(root_directory).as_posix()) for path in paths)

    return flask.jsonify(full_paths)



from http import HTTPStatus

import flask

from btviewer.blueprints.photo.model import Photo

app: flask.Flask = flask.current_app
blueprint = flask.Blueprint("label", __name__, url_prefix="/labels")


@blueprint.route("/detail", methods=["GET"])
def detail():
    """
    Get all the tags associated with this image

    Usage:
    /label/detail?path=1970-01-01/set_A/device_1234/camera_1/20200101_094359.123456_000002.np

    :returns: List of labels
    [
      {"confidence": "Unsure", "x": 321, "y": 789},
      {"confidence": "Sure", "x": 123, "y": 6564},
      {"confidence": "Unsure", "x": 456, "y": 789}
    ]
    """
    photo = Photo(flask.request.args["path"])
    return flask.jsonify(photo.labels)


@blueprint.route("/create", methods=["POST"])
def create():
    """
    Create new labels

    The front end will send a POST request to create the labels with an
    array of label information like so:

    POST
    http://localhost:5000/labels/create?path=1970-01-01/set_A/device_1234/camera_1/20200101_094359.123456_000002.np

    [
        {
          "x": 123,
          "y": 456,
          "confident": true,
          "label": "not the bees!"
        },
        {
          "x": 123,
          "y": 456,
          "confident": true,
          "label": "To be, or not to be."
        }
    ]
    """

    # Load the selected image
    photo_path = flask.request.args["path"]
    photo = Photo(photo_path)

    # Get the label data from the request
    source = flask.request.args["source"]
    version = flask.request.args["version"]
    labels = flask.request.json

    # Create the labels
    label_path = photo.add_labels(labels, source=source, version=version)

    # Return a success response
    return flask.jsonify(dict(label_path=str(label_path))), HTTPStatus.CREATED


@blueprint.route("/delete", methods=["DELETE"])
def delete():
    """
    Delete all the labels on a photo by deleting the file
    """

    # Load the selected image
    photo_path = flask.request.args["path"]
    photo = Photo(photo_path)

    # Delete all photos from this course
    source = flask.request.args["source"]
    photo.delete_labels(source)

    return HTTPStatus.NO_CONTENT


# the modify route is for deleting single label as the methods used is post, not delete compared to the delete all.
@blueprint.route("/modify", methods=["POST"])
def modify():
    """
    Delete single label on a photo
    """

    # Load the selected image
    photo = Photo(flask.request.args["path"])
    source = flask.request.args["source"]
    x = int(flask.request.args["x"])
    y = int(flask.request.args["y"])

    photo.delete_labels(source, x, y)

    return HTTPStatus.NO_CONTENT


@blueprint.route("/annotate", methods=["POST"])  # the annotate route is for annotate existing single label
def annotate():
    """
    annotate an existing label on a photo
    """

    # Load the selected image
    photo = Photo(flask.request.args["path"])
    source = flask.request.args["source"]
    x = int(flask.request.args["x"])
    y = int(flask.request.args["y"])
    annotation = flask.request.args["annotation"]

    photo.annotate_labels(source, annotation, x, y)

    return HTTPStatus.NO_CONTENT



#!/usr/bin/env python

import argparse
import logging
import os
import webbrowser
from pathlib import Path

import btviewer.app_factory
import waitress

DESCRIPTION = """
This is a browser-based app for viewing and human labelling of tracking images.
Example usage: btviewer ~/path/to/data/
"""


def get_args() -> argparse.Namespace:
    """
    Configure command-line arguments.
    """

    parser = argparse.ArgumentParser(description=DESCRIPTION)

    # Actions
    # https://docs.python.org/dev/library/argparse.html#action
    parser.add_argument('-v', '--verbose', action='store_true')
    parser.add_argument('--version', action='version', version=f'%(prog)s {btviewer.__version__}')

    # Flask options
    parser.add_argument('--debug', type=bool, default=True)
    parser.add_argument('--host', default='127.0.0.1')
    parser.add_argument('--port', default=5000)
    parser.add_argument('--threads', default=os.cpu_count(), type=int)

    # btviewer options
    parser.add_argument('root_directory', type=Path)

    return parser.parse_args()


def main():
    args = get_args()
    logging.basicConfig(level=logging.DEBUG if args.verbose else logging.INFO)

    # Create WSGI app
    app = btviewer.app_factory.create_app(root_directory=args.root_directory)

    logging.info("Data root directory: %s", app.config['ROOT_DIRECTORY'])

    # Get URI of backend
    uri = f"http://{args.host}:{args.port}"
    logging.info(f'Running backend with {args.threads} threads')

    # Open frontend in web browser
    static_uri = uri + '/static/index.html'
    webbrowser.open(static_uri)

    # Run web server
    # https://docs.pylonsproject.org/projects/waitress/en/latest/arguments.html
    waitress.serve(app, host=args.host, port=args.port, threads=args.threads)


if __name__ == '__main__':
    main()



"""
Flask application factory
"""
import os
from pathlib import Path
from typing import Union

import flask


def create_app(root_directory: Union[Path, str] = None, **kwargs) -> flask.Flask:
    """
    Build the Flask app instance

    This function is a Flask application factory
    https://flask.palletsprojects.com/en/3.0.x/patterns/appfactories/

    :param root_directory: The base directory for image capture session data.
    :param kwargs: keyword arguments for the Flask app
    """
    app = flask.Flask(__name__, **kwargs)
    root_directory = Path(root_directory or os.getenv('ROOT_DIRECTORY', '.')).absolute()
    app.config.update(dict(
        ROOT_DIRECTORY=str(root_directory),
    ))
    register_blueprints(app)

    app.after_request(set_headers)

    return app


def set_headers(response: flask.Response):
    response.headers['Access-Control-Allow-Origin'] = '*'
    return response


def register_blueprints(app: flask.Flask):
    """
    Add the blueprints to the flask app
    https://flask.palletsprojects.com/en/3.0.x/blueprints/
    """

    import btviewer.blueprints.session.views
    import btviewer.blueprints.label.views
    import btviewer.blueprints.photo.views

    app.register_blueprint(btviewer.blueprints.photo.views.blueprint)
    app.register_blueprint(btviewer.blueprints.label.views.blueprint)
    app.register_blueprint(btviewer.blueprints.session.views.blueprint)



import datetime
import io
import json
from pathlib import Path
from typing import Iterable, Mapping, Union, Generator

import PIL.Image
import flask
import numpy

app: flask.Flask = flask.current_app

Coordinate = tuple[int, int]
"x, y position in a 2D image pixel grid"


class Photo:
    """
    An image taken by a camera.
    """

    def __init__(self, path: Union[Path, str]):
        self.path = Path(app.config['ROOT_DIRECTORY']).joinpath(path).absolute()

    @classmethod  # not used, to be deleted
    def validate_filename(cls, filename: str):
        """
        Validate a filename <timestamp>_<photo_id>.np
        https://github.com/SheffieldMLtracking#file-structures
        where timestamp is YYYMMDD_HHMMSS.UUUUUU
        where photo_id is a zero-padded integer
        """
        filename = Path(filename)
        if filename.suffix != ".np":
            raise ValueError(filename)
        timestamp, _, photo_id = filename.stem.rpartition("_")
        photo_id = int(photo_id)
        cls.parse_timestamp(timestamp)

    @classmethod  # only used in validate_filename, to be deleted.
    def parse_timestamp(cls, timestamp: str) -> datetime.datetime:
        """
        Parse the `timestamp` part of a photo filename.
        https://github.com/SheffieldMLtracking#file-structures
        "YYYMMDD_HHMMSS.UUUUUU" -> datetime.datetime()
        """
        # https://docs.python.org/3/library/datetime.html#strftime-and-strptime-behavior
        dt = datetime.datetime.strptime(timestamp, "%Y%m%d_%H%M%S.%f")
        # Use UTC timestamp
        dt.replace(tzinfo=datetime.timezone.utc)
        return dt

    @classmethod
    def root_directory(cls) -> Path:
        return Path(app.config['ROOT_DIRECTORY']).absolute()

    @property  # not used
    def timestamp_string(self) -> str:
        raise NotImplementedError

    @property  # not used
    def filename(self) -> str:
        return f"{self.timestamp_string}_{self.photo_id}.np"

    def load(self) -> dict:
        """
        Get the image data from the storage disk.
        :returns:
        {
          "record": {...},
          "image": array(...)
        }
        """
        # https://numpy.org/doc/stable/reference/generated/numpy.load.html
        data = numpy.load(self.path, allow_pickle=True)
        app.logger.info("Loaded '%s'", self.path)
        return data

    def add_label(self, label: dict, **kwargs):  # Not used
        """
        Add a label to the image.

        :param label: The tag info
        :param kwargs: Label parameters
        :return:
        """
        return self.add_labels([label], **kwargs)

    def add_labels(self, labels: list[dict], source: str, version: str, indent: int = 2) -> Path:
        label_path = self.make_label_path(source=source)

        # Build a list of labels, starting with an empty list
        document: list[dict] = []

        # Open existing labels file
        try:
            with label_path.open('r') as file:
                document.extend(json.load(file))
        except FileNotFoundError:
            pass

        # Append the metadata to each new label
        metadata = {
            "source": source,
            "version": version,
            "mode": "manual",
            "annotation": ""
        }
        for label in labels:
            label.update(metadata)

        # Include the newly-added labels
        document.extend(labels)

        # Save labels to disk
        with label_path.open('w') as file:
            json.dump(document, file)

        app.logger.info("Labels saved to '%s'", label_path)
        return label_path

    @property
    def label_filename(self) -> str:
        """
        The filename of each label file

        e.g. "2020-01-01T09+40+43_00123.json"
        """
        return self.path.with_suffix('.json').name

    def make_label_path(self, source: str) -> Path:
        """
        The path of the file containing all the label files for this photo for this source.
        """
        # If the photo path is
        # ~/photos/2020-01-01T09+40+43_00123.np
        # the label directory for btviewer is
        # ~/photos/btviewer/2020-01-01T09+40+43_00123.json

        # Make a subdirectory for this source
        label_dir = self.path.parent.joinpath(source)
        label_dir.mkdir(exist_ok=True)

        # Build the label path
        label_path = label_dir.joinpath(self.label_filename)

        return label_path

    @property
    def metadata(self) -> dict:
        """
        Get all the image information, except the 2D image data array.
        """
        return {key: value for key, value in self.data.items() if key != 'img'}

    @property
    def dimension(self) -> dict:
        """
        Get all the image dimension
        """
        image = self.data['img']
        width = len(image[0])
        height = len(image)
        return {'width': width, 'height': height}

    def to_tiff(self):
        """
        Convert image data to TIFF format
        """
        return self.to_bytes(format='TIFF')

    def to_jpeg(self):
        return self.to_bytes(format='JPEG')

    @property
    def data(self) -> Mapping:
        """
        Image data and metadata
        """
        return self.load()

    @property
    def array(self):
        return self.get_array()

    def get_array(self, scale_factor: float = None, dtype=numpy.dtype('uint8'),
                  mean_exposure: float = 0.18) -> numpy.array:
        """
        2D image data array of pixel values.

        By default, this produces a normalised 2D grid of unsigned 8-bit integers (0 to 255).

        :param scale_factor: Multiplication factor for the pixel brightness values
        :param dtype: Data type https://numpy.org/doc/stable/reference/generated/numpy.ndarray.astype.html
        :param mean_exposure: The target average brightness (default 1to 18% grey)
        """
        # Set data type for array values
        dtype = numpy.dtype(dtype)
        maximum_value = numpy.iinfo(dtype).max  # machine limits for integer type

        # Load image data
        array: numpy.ndarray = self.data['img']

        if scale_factor is None:
            # Adjust brightness to target average exposure
            scale_factor = maximum_value * mean_exposure / array.mean()

        numpy.multiply(array, scale_factor, out=array, casting='unsafe')

        return array.astype(dtype=dtype)

    @property
    def image(self) -> PIL.Image:
        """
        A PIL image object for the photo data.
        """
        return PIL.Image.fromarray(self.array)

    def to_bytes(self, **kwargs) -> io.BytesIO:
        """
        Convert the 2D image data to an image file format.

        :param kwargs: Keyword arguments for PIL Image.save()
        :returns: Image data buffer
        """
        # Use BytesIO to store the image in memory
        buffer = io.BytesIO()
        self.image.save(buffer, **kwargs)
        buffer.seek(0)

        return buffer

    def to_png(self):
        """
        Convert image data to PNG format
        """
        return self.to_bytes(format='PNG')

    def iter_labels(self) -> Generator[Mapping, None, None]:
        """
        Iterate over label documents for this image.
        """
        # Iterate over subdirectories 
        path: Path
        for path in self.path.parent.iterdir():
            # Subdirectories only, for example btviewer folder and retrodetect
            if path.is_dir():
                label_path = path.joinpath(self.label_filename)  # btviewer/photofilename.json
                try:
                    # Load JSON data
                    with label_path.open() as file:
                        doc = json.load(file)
                        app.logger.info("Loaded '%s'", file.name)
                    # Get list of tags
                    yield from doc  # https://www.geeksforgeeks.org/use-yield-keyword-instead-return-keyword-python/
                except FileNotFoundError:
                    continue

    @property
    def labels(self) -> Iterable[Mapping]:
        """
        The tags applied to this image.
        """
        return list(dict(label) for label in self.iter_labels())

    def next(self, skip: int = 1) -> Path:
        """
        Get the path of the next file in this directory.

        :param skip: The number of photo files to navigate through.
        """

        # Get the list of photo filenames in the current directory
        camera_dir = self.path.parent
        photos = sorted(camera_dir.glob('*.np'))

        # Figure out the current position in that list
        current_index = photos.index(self.path)

        # Go to the previous/next file
        # Skip x photos (skip may be negative or positive)
        try:
            return photos[current_index + skip]
        except IndexError:
            # If there are no more,. just go the first one
            return photos[0]

    def delete_labels(self, source: str, x: int = None, y: int = None, indent: int = 2):
        """
        Delete some labels associated with this photo.

        :param source: The app that created the label e.g. "btviewer"
        :param x: The horizontal pixel coordinate
        :param y: The vertical pixel coordinate
        :param indent: JSON formatting option
        """

        # Get the label file
        label_path = self.make_label_path(source=source)

        # Delete all labels from that source when no specific label specified
        if not x:

            label_path.unlink(missing_ok=True)
            app.logger.info("Deleted '%s'", label_path)

        # Delete a specific label
        else:
            with label_path.open() as file:
                labels = json.load(file)

            # Exclude the coordinates to be deleted
            labels = [label for label in labels if int(label['x']) != int(x) and int(label['y']) != int(y)]

            # Save changes to disk
            with label_path.open('w') as file:
                json.dump(labels, file, indent=indent)
                app.logger.info("Deleted label at %s, %s from '%s'", x, y, file.name)

    def annotate_labels(self, source: str, annotation_text: str, x: int, y: int, indent: int = 2, ):
        """
        Add annotation to existing labels associated with this photo.

        :param source: The app that created the label e.g. "btviewer"
        :param x: The horizontal pixel coordinate
        :param y: The vertical pixel coordinate
        :param indent: JSON formatting option
        :param annotation_text: annotation text
        """
        # Get the label file
        label_path = self.make_label_path(source=source)
        with label_path.open() as file:
            labels = json.load(file)

        # find the label corresponding to the clicked point to add the annotation
        # Note it will replace existing annotation
        for label in labels:
            if int(label['x']) == int(x) and int(label['y']) == int(y):
                label.update({"annotation": annotation_text})

        # Save changes to disk
        with label_path.open('w') as file:
            json.dump(labels, file, indent=indent)
            app.logger.info("annotate label at %s, %s from '%s'", x, y, file.name)

    def select_region_of_interest(self, box: tuple[Coordinate, Coordinate]) -> numpy.ndarray:
        """
        Select an area within the 2D pixel array of the photo image.

        :param box: The coordinates of the top-left and bottom-right pixel of the selected 2D region of interest.

        top_left, ...
        ..., bottom_right
        """

        # Load the image data
        array = self.array

        # Get the two corner coordinates
        top_left, bottom_right = box

        # Get the pixel values inside the bounding box only
        region_of_interest = array[top_left[0]: bottom_right[0], top_left[1]: bottom_right[1]]

        return region_of_interest

    def find_brightest_pixel(self, box: tuple[Coordinate, Coordinate]) -> Coordinate:
        """
        Get the coordinate of the brightest pixel in the specified bounding area.

        :param box: The coordinates of the top-left and bottom-right pixel of the selected 2D region of interest.
        """

        region_of_interest = self.select_region_of_interest(box=box)

        # Get the position of the  brightest pixel (using a flat index)
        flat_max_index = numpy.argmax(region_of_interest)

        # Get the relative coordinates of that brightest pixel (inside the ROI)
        x, y = numpy.unravel_index(flat_max_index, region_of_interest.shape)

        # Get the absolute coordinates (relative to the entire image)
        top_left_coordinate = box[0]
        x += top_left_coordinate[0]
        y += top_left_coordinate[1]

        # Cast from numpy.int64 to integer
        return int(x), int(y)



from http import HTTPStatus

import numpy as np

from btviewer.blueprints.photo.model import Photo

PHOTO_PATH = '2020-01-01/set_A/device_1/camera_1/20200101_094359.123456_000001.np'


def test_photo_array(app_context):
    photo = Photo(PHOTO_PATH)

    assert isinstance(photo.array, np.ndarray)
    assert photo.array.shape == (1536, 2048)
    assert photo.array.min() >= 0
    assert photo.array.max() <= 255


def test_photo_jpeg(client):
    width = 1024
    height = 768
    response = client.get('photos/2020-01-01/set_A/device_1/camera_1/20200101_094359.123456_000001.jpeg')
    assert response.status_code == HTTPStatus.OK

    # Check response contents
    assert response.content_type == 'image/jpeg'

    # TODO check dimensions
    # import PIL.Image
    # PIL.Image.frombytes(mode='L', size=(width, height), dataresponse.data)


def test_select_region_of_interest(app_context):
    photo = Photo(PHOTO_PATH)

    # Define region of interest
    x0 = 10
    y0 = x0 + 10
    box = ((x0, x0), (y0, y0))

    region_of_interest = photo.select_region_of_interest(box)

    assert isinstance(region_of_interest, np.ndarray)
    assert region_of_interest.shape == (y0 - x0, y0 - x0)


def test_find_brightest_pixel(app_context):
    photo = Photo(PHOTO_PATH)

    # Define region of interest
    x0 = 10
    y0 = x0 + 10
    box = ((x0, x0), (y0, y0))

    # Run the code
    x, y = photo.find_brightest_pixel(box=box)

    # Check data type
    assert isinstance(x, int)
    assert isinstance(y, int)

    # Check pixel position
    assert x0 <= x <= y0
    assert x0 <= y <= y0



import pathlib


class StructureMixin:
    """
    These behaviours are common to every part of the file structure
    """

    # defines the default string representation of an object of a class inheriting from StructureMixin. It simply returns the object's id
    def __str__(self):
        return self.id

    # defines the "official" string representation, used when you print the object or use repr()
    def __repr__(self):
        return f"{self.__class__.__name__}('{self.id}')"

    
    @property
    def path(self) -> pathlib.Path:
        raise NotImplementedError

    @property
    def uri(self) -> str:
        return self.path.as_uri()



"""
Flask app to run in development mode
"""

import os
from pathlib import Path

from .app_factory import create_app

# Use test data
ROOT_DIRECTORY = os.getenv('ROOT_DIRECTORY', Path(__file__).parent.parent.joinpath('tests/data'))

app = create_app(root_directory=ROOT_DIRECTORY)
